<?php
/**
 * StoreApps Cache
 *
 * @category    Class
 * @package     StoreApps Connector
 * @author      StoreApps
 * @version     1.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'StoreApps_Cache' ) ) {

	// @codingStandardsIgnoreStart //
	/*
	// Example usage
	$cache = new StoreApps_Cache('category', 14 * 86400);
	$val = $cache->get( $key );
	if ( $val === null ) {
		// Not in cache, compute
		$val = doyourprocess();
		$cache->set( $key, $val );
	}
	// Now use the $val

	function cleanup() {
		$cache = new StoreApps_Cache('category', 14 * 86400);
		$cache->cleanup();
	}

	// For cleanup
	wp_schedule_event( time(), 'daily', array($this, 'cleanup') );
	*/
	// @codingStandardsIgnoreEnd //
	/**
	 * Main class for StoreApps Cache
	 */
	class StoreApps_Cache {

		public $base_dir;
		public $expire_after;
		public $enabled;
		public $_group;
		public $_hashes;

		public function set( $key, $data ) {
			if ( ! $this->enabled ) {
				return false;
			}
			$res = file_put_contents( $this->_file( $key ), wp_json_encode( $data ) ); // phpcs:ignore
			return ( false === $res ) ? false : true;
		}

		public function get( $key ) {
			if ( ! $this->enabled ) {
				return null;
			}

			if ( $this->_exists( $key ) ) {
				return json_decode( file_get_contents( $this->_file( $key ) ) ); // phpcs:ignore
			}
			return null;
		}

		public function delete( $key ) {
			if ( ! $this->enabled ) {
				return true;
			}
			if ( $this->_exists( $key ) ) {
				return unlink( $this->_file( $key ) ); // phpcs:ignore
			}
			return true;
		}

		public function cleanup() {
			if ( ! $this->enabled ) {
				return true;
			}

			foreach ( glob( $this->base_dir . $this->_group . '*' ) as $filename ) {
				if ( filemtime( $filename ) < time() - $this->expire_after ) {
					@unlink( $filename ); // phpcs:ignore
				}
			}
			return true;
		}

		private function _exists( $key ) {
			return ( is_file( $this->_file( $key ) ) );
		}

		private function _file( $key ) {
			return $this->base_dir . $this->_group . '_' . $this->_hash( $key );
		}

		private function _hash( $key ) {
			if ( ! array_key_exists( $key, $this->_hashes ) ) {
				$this->_hashes[ $key ] = md5( $key );
			}
			return $this->_hashes[ $key ];
		}

		public function __construct( $group = '', $expire_after = 86400, $base_dir = '' ) {
			$this->_group       = sanitize_key( $group );
			$this->base_dir     = $base_dir;
			$this->expire_after = $expire_after;
			$this->_hashes      = array();

			if ( empty( $this->base_dir ) ) {
				$uploads          = wp_upload_dir();
				$uploads_base_dir = trailingslashit( $uploads['basedir'] );
				$this->base_dir   = $uploads_base_dir . 'sacache/';
			}

			if ( ! is_dir( $this->base_dir ) ) {
				if ( false === mkdir( $this->base_dir ) ) { // phpcs:ignore
					$this->enabled = false;
					return;
				}
			}
			$this->base_dir = trailingslashit( $this->base_dir );
			$this->enabled  = true;
		}
	}

}
